@@ -3,16 +3,20 @@ use std::fmt::Debug;
 use std::marker::PhantomData;
 use std::ops::Range;
 
+use super::dual_coord::DualCoordChartContext;
 use super::mesh::MeshStyle;
 use super::series::SeriesLabelStyle;
 
-use crate::coord::{CoordTranslate, MeshLine, Ranged, RangedCoord, ReverseCoordTranslate, Shift};
+use crate::coord::{
+    AsRangedCoord, CoordTranslate, MeshLine, Ranged, RangedCoord, ReverseCoordTranslate, Shift,
+};
 use crate::drawing::backend::{BackendCoord, DrawingBackend};
 use crate::drawing::{DrawingArea, DrawingAreaErrorKind};
 use crate::element::{Drawable, DynElement, IntoDynElement, Path, PointCollection};
 use crate::style::{FontTransform, ShapeStyle, TextStyle};
 
 /// The annotations (such as the label of the series, the legend element, etc)
+#[allow(clippy::type_complexity)]
 pub struct SeriesAnno<'a, DB: DrawingBackend> {
     label: Option<String>,
     draw_func: Option<Box<dyn Fn(BackendCoord) -> DynElement<'a, DB, BackendCoord> + 'a>>,
@@ -62,8 +66,8 @@ impl<'a, DB: DrawingBackend> SeriesAnno<'a, DB> {
 /// Any plot/chart is abstracted as this type, and any data series can be placed to the chart
 /// context.
 pub struct ChartContext<'a, DB: DrawingBackend, CT: CoordTranslate> {
-    pub(super) x_label_area: Option<DrawingArea<DB, Shift>>,
-    pub(super) y_label_area: Option<DrawingArea<DB, Shift>>,
+    pub(super) x_label_area: [Option<DrawingArea<DB, Shift>>; 2],
+    pub(super) y_label_area: [Option<DrawingArea<DB, Shift>>; 2],
     pub(super) drawing_area: DrawingArea<DB, CT>,
     pub(super) series_anno: Vec<SeriesAnno<'a, DB>>,
 }
@@ -83,6 +87,7 @@ impl<
         MeshStyle {
             axis_style: None,
             x_label_offset: 0,
+            y_label_offset: 0,
             draw_x_mesh: true,
             draw_y_mesh: true,
             draw_x_axis: true,
@@ -140,6 +145,28 @@ impl<'a, DB: DrawingBackend, X: Ranged, Y: Ranged> ChartContext<'a, DB, RangedCo
         self.drawing_area.map_coordinate(coord)
     }
 
+    pub(super) fn draw_series_impl<E, R, S>(
+        &mut self,
+        series: S,
+    ) -> Result<(), DrawingAreaErrorKind<DB::ErrorType>>
+    where
+        for<'b> &'b E: PointCollection<'b, (X::ValueType, Y::ValueType)>,
+        E: Drawable<DB>,
+        R: Borrow<E>,
+        S: IntoIterator<Item = R>,
+    {
+        for element in series {
+            self.drawing_area.draw(element.borrow())?;
+        }
+        Ok(())
+    }
+
+    pub(super) fn alloc_series_anno(&mut self) -> &mut SeriesAnno<'a, DB> {
+        let idx = self.series_anno.len();
+        self.series_anno.push(SeriesAnno::new());
+        &mut self.series_anno[idx]
+    }
+
     /// Draw a data series. A data series in Plotters is abstracted as an iterator of elements
     pub fn draw_series<E, R, S>(
         &mut self,
@@ -151,35 +178,20 @@ impl<'a, DB: DrawingBackend, X: Ranged, Y: Ranged> ChartContext<'a, DB, RangedCo
         R: Borrow<E>,
         S: IntoIterator<Item = R>,
     {
-        for element in series {
-            self.drawing_area.draw(element.borrow())?;
-        }
-
-        let idx = self.series_anno.len();
-
-        self.series_anno.push(SeriesAnno::new());
-
-        Ok(&mut self.series_anno[idx])
+        self.draw_series_impl(series)?;
+        Ok(self.alloc_series_anno())
     }
 
-    // TODO: Remove the hardcoded size
-    #[allow(clippy::too_many_arguments)]
-    pub(super) fn draw_mesh<FmtLabel>(
+    /// The actual function that draws the mesh lines.
+    /// It also returns the label that suppose to be there.
+    #[allow(clippy::type_complexity)]
+    fn draw_mesh_lines<FmtLabel>(
         &mut self,
         (r, c): (usize, usize),
+        (x_mesh, y_mesh): (bool, bool),
         mesh_line_style: &ShapeStyle,
-        label_style: &TextStyle,
         mut fmt_label: FmtLabel,
-        x_mesh: bool,
-        y_mesh: bool,
-        x_label_offset: i32,
-        x_axis: bool,
-        y_axis: bool,
-        axis_style: &ShapeStyle,
-        axis_desc_style: &TextStyle,
-        x_desc: Option<String>,
-        y_desc: Option<String>,
-    ) -> Result<(), DrawingAreaErrorKind<DB::ErrorType>>
+    ) -> Result<(Vec<(i32, String)>, Vec<(i32, String)>), DrawingAreaErrorKind<DB::ErrorType>>
     where
         FmtLabel: FnMut(&MeshLine<X, Y>) -> Option<String>,
     {
@@ -211,81 +223,218 @@ impl<'a, DB: DrawingBackend, X: Ranged, Y: Ranged> ChartContext<'a, DB, RangedCo
             r,
             c,
         )?;
+        Ok((x_labels, y_labels))
+    }
+
+    // TODO: consider make this function less complicated
+    #[allow(clippy::cognitive_complexity)]
+    #[allow(clippy::too_many_arguments)]
+    fn draw_axis_and_labels(
+        &self,
+        area: Option<&DrawingArea<DB, Shift>>,
+        axis_style: Option<&ShapeStyle>,
+        labels: &[(i32, String)],
+        label_style: &TextStyle,
+        label_offset: i32,
+        orientation: (i16, i16),
+        axis_desc: Option<(&str, &TextStyle)>,
+    ) -> Result<(), DrawingAreaErrorKind<DB::ErrorType>> {
+        let area = if let Some(target) = area {
+            target
+        } else {
+            return Ok(());
+        };
 
         let (x0, y0) = self.drawing_area.get_base_pixel();
 
-        if let Some(ref xl) = self.x_label_area {
-            let (tw, th) = xl.dim_in_pixel();
-            if x_axis {
-                xl.draw(&Path::new(vec![(0, 0), (tw as i32, 0)], axis_style.clone()))?;
-            }
-            for (p, t) in x_labels {
-                let (w, _) = label_style.font.box_size(&t).unwrap_or((0, 0));
+        /* TODO: make this configure adjustable */
+        let knob_size = 5;
+        let label_dist = if orientation.1 > 0 { 0 } else { 10 };
 
-                if p - x0 + x_label_offset > 0 && p - x0 + x_label_offset + w as i32 / 2 < tw as i32
-                {
-                    if x_axis {
-                        xl.draw(&Path::new(
-                            vec![(p - x0, 0), (p - x0, 5)],
-                            axis_style.clone(),
-                        ))?;
-                    }
-                    xl.draw_text(
-                        &t,
-                        label_style,
-                        (p - x0 - w as i32 / 2 + x_label_offset, 10),
-                    )?;
+        let (tw, th) = area.dim_in_pixel();
+
+        let mut axis_range = if orientation.0 == 0 {
+            self.drawing_area.get_x_axis_pixel_range()
+        } else {
+            self.drawing_area.get_y_axis_pixel_range()
+        };
+
+        if orientation.0 == 0 {
+            axis_range.start -= x0;
+            axis_range.end -= x0;
+        } else {
+            axis_range.start -= y0;
+            axis_range.end -= y0;
+        }
+
+        if let Some(style) = axis_style {
+            let mut x0 = if orientation.0 > 0 { 0 } else { tw as i32 };
+            let mut y0 = if orientation.1 > 0 { 0 } else { th as i32 };
+            let mut x1 = if orientation.0 >= 0 { 0 } else { tw as i32 };
+            let mut y1 = if orientation.1 >= 0 { 0 } else { th as i32 };
+
+            if orientation.0 == 0 {
+                x0 = axis_range.start;
+                x1 = axis_range.end;
+            } else {
+                y0 = axis_range.start;
+                y1 = axis_range.end;
+            }
+            area.draw(&Path::new(vec![(x0, y0), (x1, y1)], style.clone()))?;
+        }
+
+        let right_most = if orientation.0 > 0 && orientation.1 == 0 {
+            labels
+                .iter()
+                .map(|(_, t)| label_style.font.box_size(t).unwrap_or((0, 0)).0)
+                .max()
+                .unwrap_or(0) as i32
+                + label_dist as i32
+        } else {
+            0
+        };
+
+        for (p, t) in labels {
+            let rp = if orientation.0 == 0 { *p - x0 } else { *p - y0 };
+
+            if rp < axis_range.start.min(axis_range.end)
+                || axis_range.end.max(axis_range.start) < rp
+            {
+                continue;
+            }
+
+            let (w, h) = label_style.font.box_size(&t).unwrap_or((0, 0));
+
+            let (cx, cy) = match orientation {
+                (dx, dy) if dx > 0 && dy == 0 => (right_most - w as i32, *p - y0),
+                (dx, dy) if dx < 0 && dy == 0 => (tw as i32 - label_dist - w as i32, *p - y0),
+                (dx, dy) if dx == 0 && dy > 0 => (*p - x0, label_dist + h as i32),
+                (dx, dy) if dx == 0 && dy < 0 => (*p - x0, th as i32 - label_dist - h as i32),
+                _ => panic!("Bug: Invlid orientation specification"),
+            };
+
+            let should_draw = if orientation.0 == 0 {
+                cx >= 0 && cx + label_offset + w as i32 / 2 <= tw as i32
+            } else {
+                cy >= 0 && cy + label_offset + h as i32 / 2 <= th as i32
+            };
+
+            if should_draw {
+                let (text_x, text_y) = if orientation.0 == 0 {
+                    (cx - w as i32 / 2 + label_offset, cy)
+                } else {
+                    (cx, cy - h as i32 / 2 + label_offset)
+                };
+
+                area.draw_text(&t, label_style, (text_x, text_y))?;
+
+                if let Some(style) = axis_style {
+                    let (kx0, ky0, kx1, ky1) = match orientation {
+                        (dx, dy) if dx > 0 && dy == 0 => (0, *p - y0, knob_size, *p - y0),
+                        (dx, dy) if dx < 0 && dy == 0 => {
+                            (tw as i32 - knob_size, *p - y0, tw as i32, *p - y0)
+                        }
+                        (dx, dy) if dx == 0 && dy > 0 => (*p - x0, 0, *p - x0, knob_size),
+                        (dx, dy) if dx == 0 && dy < 0 => {
+                            (*p - x0, th as i32 - knob_size, *p - x0, th as i32)
+                        }
+                        _ => panic!("Bug: Invlid orientation specification"),
+                    };
+                    let line = Path::new(vec![(kx0, ky0), (kx1, ky1)], style.clone());
+                    area.draw(&line)?;
                 }
             }
-
-            if let Some(ref text) = x_desc {
-                let (w, h) = label_style.font.box_size(text).unwrap_or((0, 0));
-
-                let left = (tw - w) / 2;
-                let top = th - h;
-
-                xl.draw_text(&text, axis_desc_style, (left as i32, top as i32))?;
-            }
         }
 
-        if let Some(ref yl) = self.y_label_area {
-            let (tw, th) = yl.dim_in_pixel();
-            if y_axis {
-                yl.draw(&Path::new(
-                    vec![(tw as i32, 0), (tw as i32, th as i32)],
-                    axis_style.clone(),
-                ))?;
-            }
-            for (p, t) in y_labels {
-                let (w, h) = label_style.font.box_size(&t).unwrap_or((0, 0));
-                if p - y0 >= 0 && p - y0 - h as i32 / 2 <= th as i32 {
-                    yl.draw_text(
-                        &t,
-                        label_style,
-                        (tw as i32 - w as i32 - 10, p - y0 - h as i32 / 2),
-                    )?;
-                    if y_axis {
-                        yl.draw(&Path::new(
-                            vec![(tw as i32 - 5, p - y0), (tw as i32, p - y0)],
-                            axis_style.clone(),
-                        ))?;
-                    }
-                }
-            }
+        if let Some((text, style)) = axis_desc {
+            let actual_style = if orientation.0 == 0 {
+                style.clone()
+            } else if orientation.0 == -1 {
+                style.transform(FontTransform::Rotate270)
+            } else {
+                style.transform(FontTransform::Rotate90)
+            };
 
-            if let Some(ref text) = y_desc {
-                let (w, _) = label_style.font.box_size(text).unwrap_or((0, 0));
+            let (w, h) = actual_style.font.box_size(text).unwrap_or((0, 0));
 
-                let top = (th - w) / 2;
+            let (x0, y0) = match orientation {
+                (dx, dy) if dx > 0 && dy == 0 => (tw - w, (th - h) / 2),
+                (dx, dy) if dx < 0 && dy == 0 => (0, (th - h) / 2),
+                (dx, dy) if dx == 0 && dy > 0 => ((tw - w) / 2, th - h),
+                (dx, dy) if dx == 0 && dy < 0 => ((tw - w) / 2, 0),
+                _ => panic!("Bug: Invlid orientation specification"),
+            };
 
-                yl.draw_text(
-                    &text,
-                    &axis_desc_style.transform(FontTransform::Rotate270),
-                    (0, top as i32),
-                )?;
-            }
+            area.draw_text(&text, &actual_style, (x0 as i32, y0 as i32))?;
         }
 
         Ok(())
     }
+
+    #[allow(clippy::too_many_arguments)]
+    pub(super) fn draw_mesh<FmtLabel>(
+        &mut self,
+        (r, c): (usize, usize),
+        mesh_line_style: &ShapeStyle,
+        label_style: &TextStyle,
+        fmt_label: FmtLabel,
+        x_mesh: bool,
+        y_mesh: bool,
+        x_label_offset: i32,
+        y_label_offset: i32,
+        x_axis: bool,
+        y_axis: bool,
+        axis_style: &ShapeStyle,
+        axis_desc_style: &TextStyle,
+        x_desc: Option<String>,
+        y_desc: Option<String>,
+    ) -> Result<(), DrawingAreaErrorKind<DB::ErrorType>>
+    where
+        FmtLabel: FnMut(&MeshLine<X, Y>) -> Option<String>,
+    {
+        let (x_labels, y_labels) =
+            self.draw_mesh_lines((r, c), (x_mesh, y_mesh), mesh_line_style, fmt_label)?;
+
+        for idx in 0..2 {
+            self.draw_axis_and_labels(
+                self.x_label_area[idx].as_ref(),
+                if x_axis { Some(axis_style) } else { None },
+                &x_labels[..],
+                label_style,
+                x_label_offset,
+                (0, -1 + idx as i16 * 2),
+                x_desc.as_ref().map(|desc| (&desc[..], axis_desc_style)),
+            )?;
+
+            self.draw_axis_and_labels(
+                self.y_label_area[idx].as_ref(),
+                if y_axis { Some(axis_style) } else { None },
+                &y_labels[..],
+                label_style,
+                y_label_offset,
+                (-1 + idx as i16 * 2, 0),
+                y_desc.as_ref().map(|desc| (&desc[..], axis_desc_style)),
+            )?;
+        }
+
+        Ok(())
+    }
+
+    /// Attach a secondary coord to the chart
+    #[allow(clippy::type_complexity)]
+    pub fn set_secondary_coord<SX: AsRangedCoord, SY: AsRangedCoord>(
+        self,
+        x_coord: SX,
+        y_coord: SY,
+    ) -> DualCoordChartContext<
+        'a,
+        DB,
+        RangedCoord<X, Y>,
+        RangedCoord<SX::CoordDescType, SY::CoordDescType>,
+    > {
+        let mut pixel_range = self.drawing_area.get_pixel_range();
+        pixel_range.1 = pixel_range.1.end..pixel_range.1.start;
+
+        DualCoordChartContext::new(self, RangedCoord::new(x_coord, y_coord, pixel_range))
+    }
 }
