@@ -8,6 +8,9 @@ macro_rules! impl_descrete_trait {
             fn next_value(this: &Self::ValueType) -> Self::ValueType {
                 return *this + 1;
             }
+            fn previous_value(this: &Self::ValueType) -> Self::ValueType {
+                return *this - 1;
+            }
         }
     };
 }
@@ -128,6 +131,7 @@ macro_rules! gen_key_points_comp {
     (integer, $name:ident, $type:ty) => {
         fn $name(range: ($type, $type), max_points: usize) -> Vec<$type> {
             let mut scale: $type = 1;
+            let range = (range.0.min(range.1), range.0.max(range.1));
             'outter: while (range.1 - range.0 + scale - 1) as usize / (scale as usize) > max_points
             {
                 let next_scale = scale * 10;
@@ -166,6 +170,8 @@ gen_key_points_comp!(integer, compute_i64_key_points, i64);
 gen_key_points_comp!(integer, compute_u64_key_points, u64);
 gen_key_points_comp!(integer, compute_i128_key_points, i128);
 gen_key_points_comp!(integer, compute_u128_key_points, u128);
+gen_key_points_comp!(integer, compute_isize_key_points, isize);
+gen_key_points_comp!(integer, compute_usize_key_points, usize);
 
 make_numeric_coord!(
     f32,
@@ -215,6 +221,18 @@ make_numeric_coord!(
     compute_i128_key_points,
     "The ranged coordinate for type i128"
 );
+make_numeric_coord!(
+    usize,
+    RangedCoordusize,
+    compute_usize_key_points,
+    "The ranged coordinate for type usize"
+);
+make_numeric_coord!(
+    isize,
+    RangedCoordisize,
+    compute_isize_key_points,
+    "The ranged coordinate for type isize"
+);
 
 impl_descrete_trait!(RangedCoordu32);
 impl_descrete_trait!(RangedCoordi32);
@@ -222,20 +240,19 @@ impl_descrete_trait!(RangedCoordu64);
 impl_descrete_trait!(RangedCoordi64);
 impl_descrete_trait!(RangedCoordu128);
 impl_descrete_trait!(RangedCoordi128);
+impl_descrete_trait!(RangedCoordusize);
+impl_descrete_trait!(RangedCoordisize);
 
 impl_ranged_type_trait!(f32, RangedCoordf32);
 impl_ranged_type_trait!(f64, RangedCoordf64);
 impl_ranged_type_trait!(i32, RangedCoordi32);
-impl_ranged_type_trait!(i64, RangedCoordi64);
 impl_ranged_type_trait!(u32, RangedCoordu32);
+impl_ranged_type_trait!(i64, RangedCoordi64);
 impl_ranged_type_trait!(u64, RangedCoordu64);
 impl_ranged_type_trait!(i128, RangedCoordi128);
 impl_ranged_type_trait!(u128, RangedCoordu128);
-
-/*
-pub fn keypoints_i64(range:(i64,i64), n:usize) -> Vec<i64> {
-    compute_i64_key_points(range,n)
-}*/
+impl_ranged_type_trait!(isize, RangedCoordisize);
+impl_ranged_type_trait!(usize, RangedCoordusize);
 
 #[cfg(test)]
 mod test {
@@ -247,6 +264,12 @@ mod test {
 
         assert!(kp.len() > 0);
         assert!(kp.len() <= 28);
+
+        let kp = compute_f64_key_points((-1.2, 1.2), 1);
+        assert!(kp.len() == 1);
+
+        let kp = compute_f64_key_points((-1.2, 1.2), 0);
+        assert!(kp.len() == 0);
     }
 
     #[test]
