@@ -1,6 +1,6 @@
 /*!
 
-# Plotters - Rust Drawing Library for Visualization  🦀 📈🚀
+# Plotters - Rust Drawing Library for Visualization  🦀📈🚀
 
 <a href="https://crates.io/crates/plotters">
     <img style="display: inline!important" src="https://img.shields.io/crates/v/plotters.svg"></img>
@@ -18,7 +18,8 @@
 Plotters is drawing library designed for rendering figures, plots, and charts, in pure rust. Plotters supports various types of backends,
 including bitmap, vector graph, piston window and WebAssembly.
 
-- You can try Plotters with Jupyter notebook, or view [here](https://plotters-rs.github.io/plotters-doc-data/evcxr-jupyter-integration.html) for the static HTML version.
+- To try Plotters with inverative Jupyter notebook, or view [here](https://plotters-rs.github.io/plotters-doc-data/evcxr-jupyter-integration.html) for the static HTML version.
+- To view the WASM example, go to this [link](https://plumberserver.com/plotters-wasm-demo/index.html)
 
 ## Gallery
 
@@ -101,6 +102,76 @@ including bitmap, vector graph, piston window and WebAssembly.
     </div>
 </div>
 
+<div class="galleryItem">
+    <a href="https://plotters-rs.github.io/plotters-doc-data/twoscale.png">
+        <img src="https://plotters-rs.github.io/plotters-doc-data/twoscale.png" class="galleryItem"></img>
+    </a>
+    <div class="galleryText">
+        Dual Y-Axis Example
+        <a href="https://github.com/38/plotters/blob/master/examples/two-scales.rs">[code]</a>
+    </div>
+</div>
+
+<div class="galleryItem">
+    <a href="https://plotters-rs.github.io/plotters-doc-data/matshow.png">
+        <img src="https://plotters-rs.github.io/plotters-doc-data/matshow.png" class="galleryItem"></img>
+    </a>
+    <div class="galleryText">
+        The matplotlib Matshow Example
+        <a href="https://github.com/38/plotters/blob/master/examples/matshow.rs">[code]</a>
+    </div>
+</div>
+
+<div class="galleryItem">
+    <a href="https://plotters-rs.github.io/plotters-doc-data/sierpinski.png">
+        <img src="https://plotters-rs.github.io/plotters-doc-data/sierpinski.png" class="galleryItem"></img>
+    </a>
+    <div class="galleryText">
+        The Sierpinski Carpet
+        <a href="https://github.com/38/plotters/blob/master/examples/sierpinski.rs">[code]</a>
+    </div>
+</div>
+
+<div class="galleryItem">
+    <a href="https://plotters-rs.github.io/plotters-doc-data/normal-dist2.png">
+        <img src="https://plotters-rs.github.io/plotters-doc-data/normal-dist2.png" class="galleryItem"></img>
+    </a>
+    <div class="galleryText">
+        The 1D Gussian Distribution
+        <a href="https://github.com/38/plotters/blob/master/examples/nomal-dist2.rs">[code]</a>
+    </div>
+</div>
+
+<div class="galleryItem">
+    <a href="https://plotters-rs.github.io/plotters-doc-data/errorbar.png">
+        <img src="https://plotters-rs.github.io/plotters-doc-data/errorbar.png" class="galleryItem"></img>
+    </a>
+    <div class="galleryText">
+        The 1D Gussian Distribution
+        <a href="https://github.com/38/plotters/blob/master/examples/errorbar.rs">[code]</a>
+    </div>
+</div>
+
+<div class="galleryItem">
+    <a href="https://plotters-rs.github.io/plotters-doc-data/slc-temp.png">
+        <img src="https://plotters-rs.github.io/plotters-doc-data/slc-temp.png" class="galleryItem"></img>
+    </a>
+    <div class="galleryText">
+        Monthly Time Coordinate
+        <a href="https://github.com/38/plotters/blob/master/examples/slc-temp.rs">[code]</a>
+    </div>
+</div>
+
+<div class="galleryItem">
+    <a href="https://plotters-rs.github.io/plotters-doc-data/area-chart.png">
+        <img src="https://plotters-rs.github.io/plotters-doc-data/area-chart.png" class="galleryItem"></img>
+    </a>
+    <div class="galleryText">
+        Monthly Time Coordinate
+        <a href="https://github.com/38/plotters/blob/master/examples/area-chart.rs">[code]</a>
+    </div>
+</div>
+
 
 ## Table of Contents
   * [Gallery](#gallery)
@@ -119,13 +190,14 @@ including bitmap, vector graph, piston window and WebAssembly.
   * [Misc](#misc)
     + [Development Version](#development-version)
     + [Reducing Depending Libraries && Turning Off Backends](#reducing-depending-libraries--turning-off-backends)
+  * [FAQ List](#faq-list)
 
 ## Quick Start
 
 To use Plotters, you can simply add Plotters into your `Cargo.toml`
 ```toml
 [dependencies]
-plotters = "^0.2.5"
+plotters = "^0.2.6"
 ```
 
 And the following code draws a quadratic function. `src/main.rs`,
@@ -478,6 +550,14 @@ This behaviour can also be turned off by setting `default_features = false`.
     }
 </style>
 
+
+## FAQ List
+
+* Why does the WASM example work on my machine ?
+
+    The WASM example requires using `wasm32` target to build. Using `cargo build` is likely to use the default target
+    which in most of the case is any of the x86 target. Thus you need add `--target=wasm32-unknown-unknown` in the cargo
+    parameter list  to build it.
 */
 pub mod chart;
 pub mod coord;
@@ -498,17 +578,21 @@ pub use palette;
 
 /// The module imports the most commonly used types and modules in Plotters
 pub mod prelude {
-    pub use crate::chart::{ChartBuilder, ChartContext, SeriesLabelPosition};
+    pub use crate::chart::{ChartBuilder, ChartContext, LabelAreaPosition, SeriesLabelPosition};
     pub use crate::coord::{
-        CoordTranslate, LogCoord, LogRange, LogScalable, Ranged, RangedCoord, RangedCoordf32,
-        RangedCoordf64, RangedCoordi32, RangedCoordi64, RangedCoordu32, RangedCoordu64,
+        CoordTranslate, IntoCentric, IntoPartialAxis, LogCoord, LogRange, LogScalable, Ranged,
+        RangedCoord, RangedCoordf32, RangedCoordf64, RangedCoordi32, RangedCoordi64,
+        RangedCoordu32, RangedCoordu64,
     };
 
     #[cfg(feature = "chrono")]
     pub use crate::coord::{RangedDate, RangedDateTime};
 
+    #[cfg(feature = "make_partial_axis")]
+    pub use crate::coord::make_partial_axis;
+
     pub use crate::drawing::*;
-    pub use crate::series::{Histogram, LineSeries, PointSeries};
+    pub use crate::series::{AreaSeries, Histogram, LineSeries, PointSeries};
     pub use crate::style::{
         Color, FontDesc, FontTransform, HSLColor, IntoFont, Palette, Palette100, Palette99,
         Palette9999, PaletteColor, RGBColor, ShapeStyle, SimpleColor, TextStyle,
@@ -516,8 +600,8 @@ pub mod prelude {
     pub use crate::style::{BLACK, BLUE, CYAN, GREEN, MAGENTA, RED, TRANSPARENT, WHITE, YELLOW};
 
     pub use crate::element::{
-        CandleStick, Circle, Cross, DynElement, EmptyElement, IntoDynElement, MultiLineText, Path,
-        Pixel, Rectangle, Text,
+        CandleStick, Circle, Cross, DynElement, EmptyElement, ErrorBar, IntoDynElement,
+        MultiLineText, Path, Pixel, Polygon, Rectangle, Text, TriangleMarker,
     };
 
     #[allow(type_alias_bounds)]
